/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistry.servlet;

import eu.europa.ec.re3gistry.constants.Constants;
import eu.europa.ec.re3gistry.module.Module;
import eu.europa.ec.re3gistry.module.ModuleMgr;
import eu.europa.ec.re3gistry.utils.Application;
import eu.europa.ec.re3gistry.utils.locale.LocalizationMgr;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;
import eu.cec.digit.ecas.client.jaas.DetailedUser;
import eu.europa.ec.re3gistrycommon.model.Users;
import eu.europa.ec.re3gistrycommon.utils.PersistenceFactory;
import javax.persistence.EntityManager;
import eu.europa.ec.re3gistrycommon.managers.UserMgr;
import javax.servlet.http.HttpSession;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;

/**
 *
 * @author francdb
 */
public class Frontend extends HttpServlet {

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code>
     * methods.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        response.setContentType(Constants.CONTENT_TYPE);
        request.setAttribute(Constants.ERROR_KEY, null);

        //Inizializing the logger
        Logger log = Logger.getLogger(Constants.BASEAPP_NAME);
        request.setAttribute(Constants.LOG_KEY, log);

        try {
            /**
             * Initializing elements *
             */
            //Initializes the properties related to the main module
            Properties props = Application.getProperties();
            request.setAttribute(Constants.PROPS_KEY, props);

            //Initializing baseurl
            String baseurl = Application.getBasepath(request);
            request.setAttribute(Constants.BASEURL_KEY, baseurl);

            //Initializint modules
            ArrayList<Module> modules = ModuleMgr.getModules(getServletContext(), request);
            request.setAttribute(Constants.MODULES_KEY, modules);

            //Initializing resource bundle (localization)
            ResourceBundle localization = ResourceBundle.getBundle(Constants.LOCALIZATION_PREFIX + Constants.BASEAPP_NAME + Constants.LOCALIZATION_SUFFIX, LocalizationMgr.getCurrentLocale(request));
            request.setAttribute(Constants.LOCALIZATION_KEY, localization);

            //Setting the current locale
            Locale currentLocale = LocalizationMgr.getCurrentLocale(request);
            request.setAttribute(Constants.CURRENTLOCALE_KEY, currentLocale);

            //Setting the available language
            List availableLanguages = LocalizationMgr.getAvailableLanguages();
            request.setAttribute(Constants.AVAILABLELANGUAGES_KEY, availableLanguages);

            /**
             * Loading request parameters *
             */
            //Module parameter
            String moduleParam = request.getParameter(Constants.MODULE_PARAMETER_KEY);
            if (moduleParam != null) {

                try {
                    //Loading the module
                    Module module = new Module(moduleParam, currentLocale);
                    request.setAttribute(Constants.MODULE_KEY, module);

                } catch (Exception e) {
                    log.error("The requested module was not found or there was an error loading it.");
                    request.setAttribute(Constants.ERROR_KEY, Constants.ERROR_NO_MODULE);
                }

            }

            EntityManager em = PersistenceFactory.getEntityManagerFactory().createEntityManager();

            //Get the current Login Type from the configuration
            String loginType = props.getProperty(Constants.LOGIN_TYPE_PROPERTY_KEY, Constants.LOGIN_TYPE_SHIRO);

            Users user = null;
            HttpSession s = request.getSession();
            UserMgr userMgr = new UserMgr(em);
            //Checking the Re3gistry user in session (user from ecas remapped in Re3gistry user).
            user = (Users) s.getAttribute(Constants.USER_KEY);
            boolean error = false;

            //Handring ECAS login type
            if (loginType.equals(Constants.LOGIN_TYPE_ECAS)) {

                //// Check user from the database ////
                DetailedUser detailedUser = (DetailedUser) request.getUserPrincipal();

                if (user == null) {
                    if (detailedUser != null) {

                        //Getting the user from the system using ECAS email as key.
                        //If the user whith that email is not available in the system,
                        //the user will be created in the database.
                        try {
                            user = userMgr.checkUserByEmail(detailedUser.getEmail());
                        } catch (Exception e) {
                            //If some problems ocurs during the checking/creation of the user
                            //An error is raised.
                            request.setAttribute(Constants.AUTH_ERROR_KEY, 1);
                            request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                            error = true;
                        }

                        //If no error has occurred in the previous operations
                        if (!error) {
                            //If the user is enabled, redirect to the application
                            if (user != null && user.getEnable()) {
                                //Setting Re3gistry user in session
                                s.setAttribute(Constants.USER_KEY, user);
                                request.getRequestDispatcher(Constants.PAGE_FORWARD).forward(request, response);
                                //Else, redirect to the login page with an error;
                            } else {
                                request.setAttribute(Constants.AUTH_INACTIVE_ERROR_KEY, 1);
                                request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                            }
                        }

                    } else {
                        //If no user is available, redirect to the login page
                        request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                    }
                } else {
                    request.getRequestDispatcher(Constants.PAGE_FORWARD).forward(request, response);
                }
            } else if (loginType.equals(Constants.LOGIN_TYPE_SHIRO)) {

                Subject detailedUser = SecurityUtils.getSubject();

                if (user == null) {
                    if (detailedUser != null && detailedUser.isAuthenticated()) {

                        //Getting the user from the system using SHIRO username as key.
                        //If the user whith that username is not available in the system,
                        //the user will be created in the database.
                        try {
                            user = userMgr.checkUserByEmail((String) detailedUser.getPrincipal());
                        } catch (Exception e) {
                            //If some problems ocurs during the checking/creation of the user
                            //An error is raised.
                            request.setAttribute(Constants.AUTH_ERROR_KEY, 1);
                            request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                            error = true;
                        }

                        //If no error has occurred in the previous operations
                        if (!error) {
                            //If the user is enabled, redirect to the application
                            if (user != null && user.getEnable()) {
                                //Setting Re3gistry user in session
                                s.setAttribute(Constants.USER_KEY, user);
                                request.getRequestDispatcher(Constants.PAGE_FORWARD).forward(request, response);
                                //Else, redirect to the login page with an error;
                            } else {
                                request.setAttribute(Constants.AUTH_INACTIVE_ERROR_KEY, 1);
                                request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                            }
                        }

                    } else {
                        //If no user is available, redirect to the login page
                        request.getRequestDispatcher(Constants.AUTH_FORWARD).forward(request, response);
                    }
                } else {
                    request.getRequestDispatcher(Constants.PAGE_FORWARD).forward(request, response);
                }
            } else {
                throw new Exception("Configuration error: login type not properly configured.");
            }

            em.close();

        } catch (Exception e) {
            log.error("Error during the page initialization. Message: " + e.getMessage());
            request.getRequestDispatcher(Constants.ERROR_GENERIC).forward(request, response);
        } finally {
        }
    }

    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }
}
