/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.constants.Constants;
import eu.europa.ec.re3gistrycommon.model.Itemcollection;
import eu.europa.ec.re3gistrycommon.model.Item;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class ItemcollectionMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public ItemcollectionMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the itemcollection with the specified uuid.
     *
     * @param uuid the unique id of the itemcollection
     * @return the itemcollection with the specified uuid
     * @throws Exception
     */
    public Itemcollection getItemcollectionByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemcollection.findByUuid");
        q.setParameter("uuid", uuid);
        return (Itemcollection) q.getSingleResult();
    }

    public List<Itemcollection> findByItem(Item item) throws Exception {
        //Checking parameters
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemcollection.findByItem");
        q.setParameter("item", item);
        return (List<Itemcollection>) q.getResultList();
    }

    public List<Itemcollection> findByCollection(Item collection) throws Exception {
        //Checking parameters
        if (collection == null) {
            throw new Exception("The collection item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Itemcollection.findByCollection");
        q.setParameter("collection", collection);
        return (List<Itemcollection>) q.getResultList();
    }

    /**
     * Insert the specified itempredecessors in the database. Returns true if
     * the operation succeed.
     *
     * @param ip the itemcollection object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertItemcollection(Itemcollection ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemcollection passed by parameter is null.");
        }

        if (ip.getDatecreation() == null) {
            ip.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(ip);

        return true;
    }

    /**
     * Update the itemcollection specified by parameter. Returns true if the
     * operation succeed.
     *
     * @param ip the updated itemcollection (this method update the
     * itemcollection on the db with the itemcollection passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateItemcollection(Itemcollection ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemcollection passed by parameter is null.");
        }

        if (ip.getDatelastupdate() == null) {
            ip.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(ip, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(ip);

        return true;
    }

    /**
     * Delete from the database the itemcollection specified by parameter.
     * Returns true if the operation succeed.
     *
     * @param ip the itemcollection do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteItemcollection(Itemcollection ip) throws Exception {
        //Checking parameters
        if (ip == null) {
            throw new Exception("The Itemcollection passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(ip);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Itemcollection ip) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getCollection() == null) {
            throw new Exception("Collection field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Itemcollection ip, boolean isUpdate) throws Exception {
        if (ip.getUuid() == null || ip.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (ip.getItem() == null) {
            throw new Exception("Item field not available.");
        }

        if (ip.getCollection() == null) {
            throw new Exception("Collection field not available.");
        }

        if (ip.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }
        if (isUpdate && ip.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }

    public Itemcollection getLatestItemcollection(Item item) throws Exception {
        if (item == null) {
            throw new Exception("The item passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNativeQuery(Constants.SQL_LARESTVERSION_OF_COLLECTION_BY_ITEM, Itemcollection.class);
        q.setParameter(1, item.getUuid());

        Itemcollection itemcollection = null;

        try {
            itemcollection = (Itemcollection) q.getSingleResult();
        } catch (Exception e) {
            throw new Exception("Error while executing query: "+e.getMessage());
        }
        return itemcollection;
    }
}
