/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrycommon.managers;

import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;

public class RegisterMgr {

    private final EntityManager em;

    /**
     * Constructor of the class. Initialize the EntityManager with the one
     * passed by parameter.
     *
     * @param em the EntityManager instance
     */
    public RegisterMgr(EntityManager em) {
        this.em = em;
    }

    /**
     * Returns the register with the specified uuid.
     *
     * @param uuid the unique id of the register
     * @return the register with the specified uuid
     * @throws Exception
     */
    public Register getRegisterByUuid(String uuid) throws Exception {
        //Checking parameters
        if (uuid == null) {
            throw new Exception("The uuid passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Register.findByUuid");
        q.setParameter("uuid", uuid);
        return (Register) q.getSingleResult();
    }
    
     public List<Register> findAll() throws Exception {
        //Preparing query
        Query q = this.em.createNamedQuery("Register.findAll");
        return (List<Register>) q.getResultList();
    }

    public List<Register> findRegisterAfterDate(Date latestStaticizerDatestart) throws Exception {
        //Checking parameters
        if (latestStaticizerDatestart == null) {
            throw new Exception("The latestStaticizerDatestart passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Register.findRegisterAfterDate");
        q.setParameter("date", latestStaticizerDatestart);
        return (List<Register>) q.getResultList();
    }

    public List<Register> findRegisterByRegistry(Registry registry) throws Exception {
        //Checking parameters
        if (registry == null) {
            throw new Exception("The registry passed by parameter is null.");
        }

        //Preparing query
        Query q = this.em.createNamedQuery("Register.findRegisterByRegistry");
        q.setParameter("registry", registry);
        return (List<Register>) q.getResultList();
    }

    /**
     * Insert the specified registers in the database. Returns true if the
     * operation succeed.
     *
     * @param re the register object to be inserted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean insertRegister(Register re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Register passed by parameter is null.");
        }

        if (re.getDatecreation() == null) {
            re.setDatecreation(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(re);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Saving the object
        this.em.persist(re);

        return true;
    }

    /**
     * Update the register specified by parameter. Returns true if the operation
     * succeed.
     *
     * @param re the updated register (this method update the register on the db
     * with the register passed by parameter)
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean updateRegister(Register re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Register passed by parameter is null.");
        }

        if (re.getDatelastupdate() == null) {
            re.setDatelastupdate(new Date());
        }

        //Checking required fields
        this.checkMandatoryFields(re, true);

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.merge(re);

        return true;
    }

    /**
     * Delete from the database the register specified by parameter. Returns
     * true if the operation succeed.
     *
     * @param re the register do be deleted
     * @return true if the operation succeed
     * @throws Exception
     */
    public boolean deleteRegister(Register re) throws Exception {
        //Checking parameters
        if (re == null) {
            throw new Exception("The Register passed by parameter is null.");
        }

        //Checking the DB managers
        if (this.em == null) {
            throw new Exception("The persistence layer is not initialized: entyty manager is null.");
        }

        //Updating the object
        em.remove(re);

        return true;
    }

    /*### Private Methods ###*/
    private boolean checkMandatoryFields(Register re) throws Exception {
        if (re.getUuid() == null || re.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (re.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (re.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (re.getRegistry() == null) {
            throw new Exception("Registry field not available.");
        }

        if (re.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        return true;
    }

    private boolean checkMandatoryFields(Register re, boolean isUpdate) throws Exception {
        if (re.getUuid() == null || re.getUuid().trim().equals("")) {
            throw new Exception("Uuid field not available.");
        }

        if (re.getBaseuri() == null) {
            throw new Exception("Baseuri field not available.");
        }

        if (re.getUriname() == null) {
            throw new Exception("Uriname field not available.");
        }

        if (re.getRegistry() == null) {
            throw new Exception("Registry field not available.");
        }

        if (re.getDatecreation() == null) {
            throw new Exception("Datecteation field not available.");
        }

        if (isUpdate && re.getDatelastupdate() == null) {
            throw new Exception("Datelastupdate field not available.");
        }

        return true;
    }
}
