/*
 * Copyright 2010,2015 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 15/07/2016
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-dev@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 */
package eu.europa.ec.re3gistrystaticizer.marshaller.custom;

import eu.europa.ec.re3gistrycommon.managers.LanguagecodeMgr;
import eu.europa.ec.re3gistrycommon.managers.LocalizationMgr;
import eu.europa.ec.re3gistrycommon.managers.RegisterMgr;
import eu.europa.ec.re3gistrycommon.managers.RegistryMgr;
import eu.europa.ec.re3gistrycommon.managers.ReportMgr;
import eu.europa.ec.re3gistrycommon.model.Languagecode;
import eu.europa.ec.re3gistrycommon.model.Localization;
import eu.europa.ec.re3gistrycommon.model.Reference;
import eu.europa.ec.re3gistrycommon.model.Register;
import eu.europa.ec.re3gistrycommon.model.Registry;
import eu.europa.ec.re3gistrycommon.utils.StringUtils;
import eu.europa.ec.re3gistrystaticizer.StaticizeFiles;
import eu.europa.ec.re3gistrystaticizer.util.Constants;
import eu.europa.ec.re3gistrystaticizer.util.RegisterUtil;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegisterStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.RegistryStandard;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DefinitionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.DescriptionAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.LabelAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.attribute.ReferenceAttribute;
import eu.europa.ec.re3gistrystaticizer.jaxbmodelstandard.list.RegisterStandardList;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.TransformerFactoryImpl;

public class RegistryCustomFiles {

    private final EntityManager em;

    private final String folderSeparator;
    private final String folderStandardStaticizer;

    private final String folderCustomStaticizer;
    private final StaticizeFiles staticizeFiles;

    private final List<Languagecode> allLanguagecode;
    private final Languagecode masterLanguage;

    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(Constants.MODULE_NAME);
    private final ReportMgr reportMgr;

    /**
     *
     * @param staticizeFiles
     * @param em
     * @throws java.lang.Exception
     */
    public RegistryCustomFiles(StaticizeFiles staticizeFiles, EntityManager em) throws Exception {
        this.staticizeFiles = staticizeFiles;
        this.folderSeparator = staticizeFiles.getFolderSeparator();
        this.folderStandardStaticizer = staticizeFiles.getCreateStandardFiles().getFolderStandardStaticizer();
        this.folderCustomStaticizer = staticizeFiles.getCreateStandardFiles().getFolderCustomStaticizer();

        this.reportMgr = staticizeFiles.getReportMgr();
        this.em = em;

        LanguagecodeMgr languagecodeMgr = new LanguagecodeMgr(em);
        allLanguagecode = languagecodeMgr.findAll();

        masterLanguage = languagecodeMgr.getMasterLanguagecode();
    }

    public List<File> addRegistryFiles() throws Exception {
        List<File> list = new ArrayList<File>();
        if (staticizeFiles.getCreateStandardFiles().getEXTENSIONS().length != 0 && !staticizeFiles.getCreateStandardFiles().getEXTENSIONS()[0].equals("")) {
            try {
                RegistryMgr registryMgr = new RegistryMgr(em);
                List<Registry> registryList = registryMgr.findAll();

                for (Registry registry : registryList) {
                    LocalizationMgr localizationMgr = new LocalizationMgr(em);
                    for (Languagecode languagecode : allLanguagecode) {
                        String isocode = languagecode.getIsocode();
                        Localization registryLocalization;
                        try {
                            registryLocalization = localizationMgr.findByRegistryAndLanguagecode(registry, languagecode);
                        } catch (Exception ex) {
                            registryLocalization = localizationMgr.findByRegistryAndLanguagecode(registry, masterLanguage);
                            isocode = masterLanguage.getIsocode();
                        }
                        RegistryStandard registryStandard = createRegistryStandard(registry, registryLocalization, languagecode);
                        list.add(createXMLRegistryFile(registryStandard, languagecode.getIsocode()));
                    }
                }
            } catch (Exception ex) {
                String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                staticizeFiles.getReportMgr().setMailSubject(subject);
                staticizeFiles.getReportMgr().setMailBody(body);
                staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                this.reportMgr.addReport("Error in writing Standard && Custom Item Files for the Registry", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                log.info("=================================Error in writing Standard && Custom Item Files for the Registry=============================================" + ex.getMessage());
                throw new Exception("Has been encountered a problem while writing the custom item files  for the Registry. ");
            }
        }
        return list;
    }

    public RegistryStandard createRegistryStandard(Registry registry, Localization registryLocalization, Languagecode languagecodeRegistry) throws Exception {
        RegistryStandard registryStandard = new RegistryStandard();
        registryStandard.setUuid(registry.getUuid());

        registryStandard.setBaseuri(registry.getBaseuri());
        registryStandard.setUriname(registry.getUriname());
        registryStandard.setUri(registry.getBaseuri() + "/" + registry.getUriname());
        registryStandard.setUristandard(registry.getBaseuri() + "/" + registry.getUriname());

        String languageLocalization = registryLocalization.getLanguage().getIsocode();

        registryStandard.setLanguage(languagecodeRegistry);

        LabelAttribute registryLabelAttribute = new LabelAttribute();
        registryLabelAttribute.setLanguage(languageLocalization);
        registryLabelAttribute.setValue(registryLocalization.getLabel());
        registryStandard.setLabel(registryLabelAttribute);

        StringUtils stringUtils = new StringUtils();

        if (registryLocalization.getDefinition() != null) {
            DefinitionAttribute registryDefinitionAttribute = new DefinitionAttribute();
            registryDefinitionAttribute.setLanguage(languageLocalization);

            String definition = stringUtils.replaceSpecialCharacters(registryLocalization.getDefinition());
            registryDefinitionAttribute.setValue(definition);
            registryStandard.setDefinition(registryDefinitionAttribute);
        }

        if (registryLocalization.getDescription() != null) {
            DescriptionAttribute registryDescriptionAttribute = new DescriptionAttribute();
            registryDescriptionAttribute.setLanguage(languageLocalization);

            String description = stringUtils.replaceSpecialCharacters(registryLocalization.getDescription());
            registryDescriptionAttribute.setValue(description);
            registryStandard.setDescription(registryDescriptionAttribute);
        }

        LocalizationMgr localizationMgr = new LocalizationMgr(em);
        Reference registryManager = registry.getRegistrymanager();
        ReferenceAttribute registermanagerReferenceAttribute = new ReferenceAttribute();
        registermanagerReferenceAttribute.setUuid(registryManager.getUuid());
        registermanagerReferenceAttribute.setUri(registryLocalization.getUri());
        registermanagerReferenceAttribute.setEmail(registryManager.getEmail());
        registermanagerReferenceAttribute.setReftype(registryManager.getReftype());
        registermanagerReferenceAttribute.setDatecreation(registryManager.getDatecreation());
        registermanagerReferenceAttribute.setDatelastupdate(registryManager.getDatelastupdate());

        String registerLabelFromLocaliation;
        LabelAttribute labelAttribute = new LabelAttribute();
        try {
            registerLabelFromLocaliation = (localizationMgr.findByReferenceAndLanguagecode(registryManager, languagecodeRegistry)).getLabel();
            labelAttribute.setLanguage(languagecodeRegistry.getIsocode());
        } catch (Exception ex) {
            registerLabelFromLocaliation = (localizationMgr.findByReferenceAndLanguagecode(registryManager, masterLanguage)).getLabel();
            labelAttribute.setLanguage(masterLanguage.getIsocode());
        }
        labelAttribute.setValue(registerLabelFromLocaliation);
        registermanagerReferenceAttribute.setLabel(labelAttribute);
        registryStandard.setRegistrymanager(registermanagerReferenceAttribute);

        registryStandard.setDatecreation(registry.getDatecreation());
        registryStandard.setDatelastupdate(registry.getDatelastupdate());

        RegisterStandardList registerStandardList = new RegisterStandardList();
        ArrayList<RegisterStandard> registerStandardArrayList = new ArrayList<RegisterStandard>();
        RegisterMgr registerMgr = new RegisterMgr(em);
        try {
            List<Register> registerList = registerMgr.findAll();

            for (Register register : registerList) {
                Localization registerLocalization;
                try {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, languagecodeRegistry);
                } catch (Exception ex) {
                    registerLocalization = localizationMgr.findByRegisterAndLanguagecode(register, masterLanguage);
                }

                RegisterStandard registerStandard = RegisterUtil.addRegisterStandard(register, languagecodeRegistry, registerLocalization, registryLabelAttribute, registry, em);
                registerStandardArrayList.add(registerStandard);
            }
        } catch (Exception ex) {
            Logger.getLogger(RegistryCustomFiles.class.getName()).log(Level.SEVERE, null, ex);
        }
        registerStandardList.setRegisterStandard(registerStandardArrayList);
        registryStandard.setRegisterStandardList(registerStandardList);

        return registryStandard;
    }

    private File createXMLRegistryFile(RegistryStandard registryStandard, String isocode) throws Exception {
        String registryStandardFolderPath = folderStandardStaticizer;
        String registryCustomFolderPath = folderCustomStaticizer;
        File registryStandardFolder = new File(registryStandardFolderPath);
        File registryCustomFolder = new File(registryCustomFolderPath);

        if (!registryStandardFolder.exists()) {
            registryStandardFolder.mkdirs();
        }
        if (!registryCustomFolder.exists()) {
            registryCustomFolder.mkdirs();
        }

        String registry = registryStandard.getUriname();
        String registryFilePath = registryStandardFolder.getAbsolutePath() + folderSeparator + registry + "." + isocode + ".xml";
        File registryFile = new File(registryFilePath);

        try {
            /**
             * STANDARD XML
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(RegistryStandard.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            marshallerclassLanguage.marshal(registryStandard, registryFile);
            log.info("Create with success: " + registryFile);

            /**
             * CUSTOM XML, JSON, HTML, ATOM, RDF
             */
            for (String ext : staticizeFiles.getCreateStandardFiles().getEXTENSIONS()) {
                try {
                    String registryXSL = staticizeFiles.getStaticizerXSLFolder() + "/" + ext + "/" + registry + "." + ext + ".xsl";
                    String finalOutputPath = registryFile.getAbsolutePath().replace("standard", "custom").replace("xml", ext);

                    File finalOutFile = new File(finalOutputPath);
                    finalOutFile.createNewFile();
                    File transformerFile = new File(registryXSL);

                    TransformerFactoryImpl factory = new net.sf.saxon.TransformerFactoryImpl();
                    Transformer transformer = factory.newTransformer(new StreamSource(transformerFile));
                    transformer.transform(new StreamSource(registryFile), new StreamResult(finalOutFile));
                    log.info("Create with success: " + finalOutFile);
                } catch (Exception ex) {
                    String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
                    String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
                    staticizeFiles.getReportMgr().setMailSubject(subject);
                    staticizeFiles.getReportMgr().setMailBody(body);
                    staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

                    this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

                    log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
                    throw new Exception("Has been encountered a problem while writing the custom item files. ");
                }
            }
        } catch (JAXBException ex) {
            String subject = staticizeFiles.getLocalizationres().getString("export.mail.subject.error");
            String body = staticizeFiles.getLocalizationres().getString("export.mail.body.error.creation.custom.item.files").replace("{user}", staticizeFiles.getDataprocedure().getUsers().getEmail());
            staticizeFiles.getReportMgr().setMailSubject(subject);
            staticizeFiles.getReportMgr().setMailBody(body);
            staticizeFiles.getReportMgr().setStatus(eu.europa.ec.re3gistrycommon.constants.Constants.ID_IMPORTED_STATUS);

            this.reportMgr.addReport("Error in writing Standard && Custom Item Files", subject, body, Constants.REPORT_TYPE_ERROR, -1);

            log.info("=================================Error in writing Standard && Custom Item Files=============================================" + ex.getMessage());
            throw new Exception("Has been encountered a problem while writing the custom item files. ");
        }

        return registryCustomFolder;
    }
}
